<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	AuthorizeCim
* @contact 		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppAuthorizecim extends PayplansAppPayment
{
	protected $_location	= __FILE__;
	
	function isApplicable($refObject = null, $eventName='')
	{
		// return true for event onPayplansSubscriptionBeforeSave
		if($eventName == 'onPayplansSubscriptionBeforeSave'){
			return true;
		}
		
		return parent::isApplicable($refObject, $eventName);
	}
		
	public function isSupportPaymentCancellation($invoice)
	{
		if($invoice->isRecurring()){
			return true;
		}
		return false;
	}
	
	private function process($xml='', $function_type='', $payment)
    {
        $subdomain  = ($this->getAppParam('sandbox',0)) ? 'apitest' : 'api';
        $url = 'https://' . $subdomain . '.authorize.net/xml/v1/request.api';
    	
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
    	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    	curl_setopt($ch, CURLOPT_HTTPHEADER, Array("Content-Type: text/xml"));
    	curl_setopt($ch, CURLOPT_HEADER, 0);
    	curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
    	curl_setopt($ch, CURLOPT_POST, 1);
    	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        $response = curl_exec($ch);
        
        if(!empty($response))
        {
        	$res = array();
        	$response = str_replace('xmlns="AnetApi/xml/v1/schema/AnetApiSchema.xsd"', '', $response);
	        $xml = new SimpleXMLElement($response);
	        
        	if($function_type == 'create'){
        		$res['resultCode']       = (string) $xml->messages->resultCode;
		        $res['code']             = (string) $xml->messages->message->code;
		        $res['text']             = (string) $xml->messages->message->text;
		        $res['profileId']        = (int) $xml->customerProfileId;
		        $res['paymentProfileId'] = (int) $xml->customerPaymentProfileIdList->numericString;
        	}
        	elseif ($function_type == 'get'){
        		$res['resultCode']       = (string) $xml->messages->resultCode;
        		$res['code']             = (string) $xml->messages->message->code;
		        $res['text']             = (string) $xml->messages->message->text;
		        $res['profileId']        = (int) $xml->profile->customerProfileId;
		        $res['paymentProfileId'] = (int) $xml->profile->paymentProfiles->customerPaymentProfileId;
		        $res['results']          = explode(',', $xml->directResponse);
        	}
        	elseif ($function_type == 'transaction'){
        		$res['resultCode']       = (string) $xml->messages->resultCode;
        		$res['code']             = (string) $xml->messages->message->code;
		        $res['text']             = (string) $xml->messages->message->text;
		        $res['results']          = explode(',', $xml->directResponse);
        	}
        	else
        	{
        		$res['resultCode']       = (string) $xml->messages->resultCode;
        		$res['code']             = (string) $xml->messages->message->code;
		        $res['text']             = (string) $xml->messages->message->text;
        	}

            curl_close($ch);
            unset($ch);
            
            return $res;
        }
        else
        {
        	$res['resultCode'] 	= 'ConnectionFailed';
        	$res['code']		= curl_errno($ch);
        	$res['text']		= curl_error($ch);
        	
        	return $res;
        }
    }
    
    function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount  = $invoice->getTotal();
		$paymentKey = $payment->getKey();
		
		$this->assign('amount', $amount);
		$this->assign('invoice', $invoice);
		$this->assign('post_url', XiRoute::_("index.php?option=com_payplans&view=payment&task=complete&action=success&payment_key=".$paymentKey));
		$this->assign('cancel_url', XiRoute::_("index.php?option=com_payplans&view=payment&task=complete&action=cancel&payment_key=".$paymentKey));
		$this->assign('payment_key', $paymentKey);
	  	return $this->_render('form');
	}
	
	function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{		
		if($action == 'cancel'){
			return true;
		}
		
		$this->_defineCredentials();
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		//
		// Step 1 :- Create Customer Profile and Customer Payment Profile
		//
		$response = $this->createCustomerProfile($payment, $invoice, $data);
		$getApp = XiFactory::getApplication();
		if ($response['resultCode'] != 'Ok' && $response['code'] != 'I00001' )
		{
			$getApp->enqueueMessage(XiText::_($response['code'].$response['text']),'error');
			$getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));	
			return true;
		}
		// Case:- Customer profile created successfully now do your process
		$invoiceAmount = $invoice->getTotal();
			
		// Case of Free Trial 
		// Do not create Transaction Profile of user
		// (Just create transaction  and mark invoice as paid)
		if($invoiceAmount == '0.00' && $invoice->isRecurring()){
			$recurrence_count = $this->_getRecurrenceCount($invoice);
			$this->_processRecurringPayment($payment, $invoice, $invoiceAmount, $recurrence_count, $response['profileId'],  $response['paymentProfileId'], array());
				//PayplansPayment $payment, PayplansInvoice $invoice, $recurrence_count, $customerProfileId, $customerPaymentId, $record = array()
			return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
		}
			
		// Step 2 :- After successful profile creation create Transaction Profile
		//
		$transaction = $this->createCustomerProfileTransaction($invoice, $payment, $invoiceAmount, $response['profileId'], $response['paymentProfileId']);

		// Case:- Error in Transaction Profile creation. remain on same page, don't redirect to thanks page.
		if($transaction['resultCode'] == 'Error')
		{
			$getApp->enqueueMessage(XiText::_($transaction['code'].$transaction['text']),'error');
			$getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));	
			return true;	
		}
		
		// Case:- Connection Failed then show proper error message
		if($transaction['resultCode'] === 'ConnectionFailed'){
			$message 	= XiText::_('COM_PAYPLANS_LOGGER_AUTHORIZE_CIM_CURL_CONNECTION_ERROR');
			$error 		= $transaction['text'].'('.$transaction['code'].')';
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $transaction, 'PayplansPaymentFormatter', '', true);
			$getApp->enqueueMessage(XiText::_($error),'error');
	  		$getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));
		}
		
		// Case:- When no connection error and no error in transaction then complete transaction
		$record = $this->_getTransactionParams($transaction);
			
		// activate the subscription only when reposnse is ok. Else just create a 0 amount of transaction.
		$amount = 0;
		if($transaction['resultCode'] == 'Ok' && $transaction['code'] == 'I00001'){
			   $amount = $record['param9'];
		}
		
		// if same notification came more than one time
	    // Check if transaction already exists,if yes then do nothing and return
	    $txn_id         = isset($record['param6']) 				? $record['param6'] 			: 0;
	    $subscr_id		= isset($response['profileId']) 		? $response['profileId'] 		: 0;
	    $parent_txn		= isset($response['paymentProfileId']) 	? $response['paymentProfileId'] : 0;
	    $transaction  	= $this->_getExistingTransaction($invoice->getId(), $txn_id, $subscr_id, $parent_txn);
		if(!empty($transaction)){
			return true;
		}
			
		// do this for recurring payment
		$recurring = $invoice->isRecurring();
		if($recurring){
			$recurrence_count = $this->_getRecurrenceCount($invoice);
			$this->_processRecurringPayment($payment, $invoice, $amount, $recurrence_count, $response['profileId'], $response['paymentProfileId'], $record);

			return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
		}

		// do this for normal payment
		// save transaction for normal payment
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $invoice->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('amount', $amount)
					->set('gateway_txn_id', isset($record['param6']) ? $record['param6'] : 0)
					->set('gateway_subscr_id', isset($response['profileId']) ? $response['profileId'] : 0)
					->set('gateway_parent_txn', isset($response['paymentProfileId']) ? $response['paymentProfileId'] : 0)
					->set('message', 'COM_PAYPLANS_PAYMENT_APP_AUTHORIZE_CIM_PAYMENT_COMPLETED_SUCCESSFULLY')
					->set('params', PayplansHelperParam::arrayToIni($record))
					->save();
		
		// save payment record
		$payment->save();			
				
		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}
	
	private function _getRecurrenceCount(PayplansInvoice $invoice)
	{
		$count = $invoice->getRecurrenceCount();
		if(intval($count) === 0){
			return 9999;
		}
		
		$recurring = $invoice->isRecurring();
		if($recurring){
			
			// Recurrence Count For Regular Recurring Plan
       		if($recurring == PAYPLANS_RECURRING){
	       		$recurrence_count = $invoice->getRecurrenceCount();
       		}
       		// Recurrence Count For Recurring + Trial 1 Plan
       		if($recurring == PAYPLANS_RECURRING_TRIAL_1){
				$recurrence_count = $invoice->getRecurrenceCount() + 1;
       		}
       		// Recurrence Count For Recurring + Trial 2 Plan
       		if($recurring == PAYPLANS_RECURRING_TRIAL_2){
	       		$recurrence_count = $invoice->getRecurrenceCount() + 2;
       		}
		}
		return $recurrence_count;
	}
	
	private function _processRecurringPayment(PayplansPayment $payment, PayplansInvoice $invoice, $amount, $recurrence_count, $customerProfileId, $customerPaymentId, $record = array())
	{
		$recurrence_count 	= $recurrence_count - 1;
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $invoice->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('amount', $amount)
					->set('gateway_txn_id', isset($record['param6']) ? $record['param6'] : 0)
					->set('gateway_subscr_id', $customerProfileId)
					->set('gateway_parent_txn', $customerPaymentId)
					->set('message', 'COM_PAYPLANS_PAYMENT_APP_AUTHORIZE_CIM_PAYMENT_COMPLETED_SUCCESSFULLY')
					->set('params', PayplansHelperParam::arrayToIni($record))
					->save();

		// IMP: if payment arrives after certain failures then reset the failure counter
		$payment->getGatewayParams()->set('pending_recur_count',$recurrence_count);
		$payment->getGatewayParams()->set('profile_id',$customerProfileId);
		$payment->getGatewayParams()->set('connection_faliure_attempt', 0);
		$payment->getGatewayParams()->set('auth_faliure_attempt', 0);
		$payment->save();
	}
	
	public function processPayment(PayplansPayment $payment, $invoiceCount)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$invoice_count = $invoiceCount + 1;
		
		if($invoice->isRecurring())
		{
			$recurrence_count 	= $payment->getGatewayParam('pending_recur_count');
			$connection_faliure = $payment->getGatewayParam('connection_faliure_attempt', 0);
			$auth_faliure		= $payment->getGatewayParam('auth_faliure_attempt', 0);

			if($recurrence_count > 0 && ($connection_faliure < $this->getAppParam('faliure_attempt', '5')) &&  ($auth_faliure < $this->getAppParam('auth_faliure_attempt', '1')))
			{
				$this->_defineCredentials();
				$profileId = $payment->getGatewayParam('profile_id');
			
				$customerProfile = $this->getCustomerProfile($profileId, $payment);
				$amount = $invoice->getTotal($invoice_count);
				$connection_faliure++;
				$auth_faliure++;

				$transaction = $this->createCustomerProfileTransaction($invoice,$payment, $amount, $profileId, $customerProfile['paymentProfileId'], $connection_faliure, $auth_faliure);
				
				if($transaction['resultCode'] === 'ConnectionFailed'){
					$message 			= XiText::_('COM_PAYPLANS_LOGGER_AUTHORIZE_CIM_CURL_CONNECTION_ERROR');
					$log_id 			= PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $transaction, 'PayplansPaymentFormatter', '', true);
					$payment->getGatewayParams()->set('connection_faliure_attempt', $connection_faliure);
					$payment->save();
				}
				
				$record = $this->_getTransactionParams($transaction);
				// set the amount if result code is OK else jsut create the transaction.
				$amount = 0;
				if($transaction['resultCode'] == 'Ok'){
					 $amount = $record['param9'];
				}
					$this->_processRecurringPayment($payment, $invoice, $amount, $recurrence_count, $profileId, $customerProfile['paymentProfileId'], $record);
			}
		}
	}
	
	public function onPayplansPaymentTerminate(PayplansPayment $payment, $controller)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$profileId = $payment->getGatewayParam('profile_id');
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $invoice->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('gateway_txn_id', $profileId);
		
		$this->_defineCredentials();
		$customerProfile = $this->getCustomerProfile($profileId, $payment);
		
		if($customerProfile !== false)
		{
			$deleteProfile = $this->deleteCustomerPaymentProfile($payment, $profileId, $customerProfile['paymentProfileId']);
			
			if($deleteProfile !== false)
			{
				$params = new XiParameter();
				$params->set('profile_id',$profileId);
				$params->set('pending_recur_count', 0);
				$transaction->set('message', 'COM_PAYPLANS_PAYMENT_APP_AUTHORIZE_CIM_CANCEL_SUCCESS')
									->save();
				$payment->set('gateway_params', $params)
								->save();					
				parent::onPayplansPaymentTerminate($payment, $controller);
				return $this->_render('cancel_success');		
			}
			$transaction->set('message', 'COM_PAYPLANS_PAYMENT_APP_AUTHORIZE_CIM_CANCEL_ERROR')
								->save();
			return $this->_render('cancel_error');
		}
		$transaction->set('message', 'COM_PAYPLANS_PAYMENT_APP_AUTHORIZE_CIM_CANCEL_ERROR')
							->save();
		return $this->_render('cancel_error');
	}
	
	public function createCustomerProfile($payment, $invoice, $data)
    {
    	$now 		= new Xidate('now');
    	$plan_title = $invoice->getTitle(). ' Time: ('.$now->toMySQL().')';
    	
        $xml_req = '<?xml version="1.0" encoding="utf-8"?>
                      <createCustomerProfileRequest xmlns="AnetApi/xml/v1/schema/AnetApiSchema.xsd">
                          <merchantAuthentication>
                              <name>' . $this->api_login_id . '</name>
                              <transactionKey>' . $this->transaction_key . '</transactionKey>
                          </merchantAuthentication>';
            $xml_req .= '
                          <refId>'. $payment->getKey() .'</refId>';
            $xml_req .= '
                          <profile>
                              <description>'. $plan_title .'</description>
                              <email>'. $data['email'] .'</email>';
            $xml_req .= '
                              <paymentProfiles>
                                  <customerType>individual</customerType>
                                  <billTo>
                                      <firstName>'. $data['first_name'].'</firstName>
                                      <lastName>'. $data['last_name'].'</lastName>
                                      <company>'. $data['company'] .'</company>
                                      <address>'. $data['address'] .'</address>
                                      <city>'. $data['city'] .'</city>
                                      <state>'. $data['state'] .'</state>
                                      <zip>'. $data['zip'] .'</zip>
                                      <country>'. $data['country'] .'</country>
                                      <phoneNumber>'. $data['mobile'].'</phoneNumber>
                                  </billTo>
                                  <payment>';
                $xml_req .= '
                                      <creditCard>
                                          <cardNumber>'. $data['card_num'].'</cardNumber>
                                          <expirationDate>'.$data['exp_year'].'-'.str_pad($data['exp_month'], 2, '0', STR_PAD_LEFT).'</expirationDate>
                                      </creditCard>';
            $xml_req .= '
                                  </payment>
                              </paymentProfiles>';
           	$xml_req .= '</profile>
                      </createCustomerProfileRequest>';
        $response = $this->process($xml_req, 'create', $payment);
        
		
   		 if(JDEBUG){
			// When debug mode is on dump in file
			file_put_contents(JPATH_SITE.DS.'tmp'.DS.time().$invoice->getKey(), var_export($response,true), FILE_APPEND);
		}
		
		
        if ($response['resultCode'] == 'Error' || $response['resultCode'] == 'ConnectionFailed')
		{
			$message 		= XiText::_('COM_PAYPLANS_LOGGER_AUTHORIZE_CIM_ERROR_IN_CUSTOMER_PROFILE_CREATION');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
			
			$getApp = XiFactory::getApplication();
			$getApp->enqueueMessage(XiText::_($response['text'].'('.$response['code'].')'),'error');
		  	$getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));
		}
		// get the transaction instace of lib
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $invoice->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('gateway_subscr_id', isset($response['profileId']) ? $response['profileId'] : 0)
					->set('message', 'COM_PAYPLANS_LOGGER_AUTHORIZE_CIM_CUSTOMER_PROFILE_CREATED')
					->set('params', PayplansHelperParam::arrayToIni($response))
					->save();
					
		$message 				= XiText::_('COM_PAYPLANS_LOGGER_AUTHORIZE_CIM_CUSTOMER_PROFILE_CREATED');
		PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, $payment, $response, 'PayplansPaymentFormatter');
		
		return $response;
    }
    
	public function getCustomerProfile($customerProfileId, $payment)
    {
        $xml_req = '<?xml version="1.0" encoding="utf-8"?>
                      <getCustomerProfileRequest xmlns="AnetApi/xml/v1/schema/AnetApiSchema.xsd">
                          <merchantAuthentication>
                              <name>' . $this->api_login_id . '</name>
                              <transactionKey>' . $this->transaction_key . '</transactionKey>
                          </merchantAuthentication>
                          <customerProfileId>'. $customerProfileId .'</customerProfileId>
                      </getCustomerProfileRequest>';
        $response = $this->process($xml_req, 'get', $payment);
        
        if($response['resultCode'] == 'Error' || $response['resultCode'] == 'ConnectionFailed')
		{
			$message = XiText::_('COM_PAYPLANS_LOGGER_AUTHORIZE_CIM_ERROR_IN_CUSTOMER_PROFILE_DETECTION');
			$log_id = PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
			
			return false;
		}
		
		return $response;
    }
    
	public function createCustomerProfileTransaction($invoice ,PayplansPayment $payment, $amount, $customerProfileId, $customerPaymentId, $connection_faliure = 0, $auth_faliure = 0)
    {
        $xml_req = '<?xml version="1.0" encoding="utf-8"?>
                      <createCustomerProfileTransactionRequest xmlns="AnetApi/xml/v1/schema/AnetApiSchema.xsd">
                          <merchantAuthentication>
                              <name>' . $this->api_login_id . '</name>
                              <transactionKey>' . $this->transaction_key . '</transactionKey>
                          </merchantAuthentication>
                          <transaction>
                              <profileTransAuthCapture>
                                  <amount>'. $amount .'</amount>';

        $xml_req .=         '<customerProfileId>'.$customerProfileId.'</customerProfileId>
                                  <customerPaymentProfileId>'.$customerPaymentId.'</customerPaymentProfileId>
                                   <order>
                                    <invoiceNumber>'.$invoice->getId().'</invoiceNumber>
                                        <description>'.$invoice->getTitle().'</description>
                                  </order>
                              </profileTransAuthCapture>
                          </transaction>
                      </createCustomerProfileTransactionRequest>';
        $response = $this->process($xml_req, 'transaction', $payment);
        
    if(JDEBUG){
			// When debug mode is on dump in file
			file_put_contents(JPATH_SITE.DS.'tmp'.DS.time().$invoice->getKey(), var_export($response,true), FILE_APPEND);
		}
		
		
        if($response['resultCode'] == 'Error'){
        	if($response['code'] == 'E00027'){
        		$payment->getGatewayParams()->set('auth_faliure_attempt', $auth_faliure);
        	}else{
        		$payment->getGatewayParams()->set('connection_faliure_attempt', $connection_faliure);
        	}
        	
			// Remove credit card info from response        	
			$unset_keys  = array('5', '6','10', '11'. '12', '50', '51');
			foreach ($unset_keys as $key){
				unset($response['results'][$key]);
			}
			
			$message = XiText::_('COM_PAYPLANS_LOGGER_AUTHORIZE_CIM_ERROR_IN_TRANSACTION_PROFILE_CREATION');
			$log_id = PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
			
			$payment->save();
		}
		
		return $response;
    }
    
	public function deleteCustomerPaymentProfile($payment, $customerProfileId, $customerPaymentId)
    {
        $xml_req = '<?xml version="1.0" encoding="utf-8"?>
                      <deleteCustomerPaymentProfileRequest xmlns="AnetApi/xml/v1/schema/AnetApiSchema.xsd">
                          <merchantAuthentication>
                              <name>' . $this->api_login_id . '</name>
                              <transactionKey>' . $this->transaction_key . '</transactionKey>
                          </merchantAuthentication>
                          <customerProfileId>'. $customerProfileId.'</customerProfileId>
                          <customerPaymentProfileId>'.$customerPaymentId.'</customerPaymentProfileId>
                      </deleteCustomerPaymentProfileRequest>';
        $response = $this->process($xml_req, 'delete', $payment);
        
        if($response['resultCode'] == 'Error')
		{
			$message = XiText::_('COM_PAYPLANS_LOGGER_AUTHORIZE_CIM_ERROR_IN_PAYMENT_PROFILE_DELETION');
			$log_id = PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
			
			return false;
		}
		
		$payment->save();
					
		$message = XiText::_('COM_PAYPLANS_LOGGER_AUTHORIZE_CIM_CUSTOMER_PAYMENT_PROFILE_DELETED');
		$log_id = PayplansHelperLogger::log(XiLogger::LEVEL_INFO, $message, $payment, $response, 'PayplansPaymentFormatter');
		
		return $response;
    }
    
	public function getRecurrenceTime($expTime)
	{
		$expTime['year'] = isset($expTime['year']) ? intval($expTime['year']) : 0;
		$expTime['month'] = isset($expTime['month']) ? intval($expTime['month']) : 0;
		$expTime['day'] = isset($expTime['day']) ? intval($expTime['day']) : 0;
		$expTime['hour'] = isset($expTime['hour']) ? intval($expTime['hour']) : 0;
		
		// if only days are set then return days as it is
		if(!empty($expTime['hour'])){
			$hours = $expTime['hour'];
			
			if(!empty($expTime['day'])){
				$hours += $expTime['day'] * 24;
				
				if(!empty($expTime['month'])){
					$hours += $expTime['month'] * 30 * 24;
	
					if(!empty($expTime['year'])){
						$hours += $expTime['year'] * 365 * 24;
					}
				}
			}
			return array('period' => $hours, 'unit' => 'Hour', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'));
		}
		
		
		// if only days are set then return days as it is
		if(!empty($expTime['day'])){
			$days = $expTime['day'];
			
			if(!empty($expTime['month'])){
				$days += $expTime['month'] * 30;

				if(!empty($expTime['year'])){
					$days += $expTime['year'] * 365;
				}
			}
			return array('period' => $days, 'unit' => 'Day', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'));
		}
		
		// if months are set
		if(!empty($expTime['month'])){
			$month = $expTime['month'];
			
			if(!empty($expTime['year'])){
				$month += $expTime['year'] * 12 ;
			}
			
			return array('period' => $month, 'unit' => 'Month', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'));
		}
		
		// years
		if(!empty($expTime['year'])){		
			return array('period' => $expTime['year'], 'unit' => 'Year', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'));
		}
		
		return false;
	}
	
    private function _getTransactionParams($response)
    {
    	$results = $response['results'];
			
		foreach($results as $key=>$value){
			$record['param'.$key] = $value;
		}
			
		return $record;
    }
    
    public function _defineCredentials()
    {
		$this->api_login_id = $this->getAppParam('api_login_id', '');    // Add your API LOGIN ID
		$this->transaction_key = $this->getAppParam('transaction_key', ''); // Add your API transaction key
    }
}


